#include <ESP8266TimeAlarms.h>
#include <ESP8266WiFi.h>
#include <ESP8266Webhook.h>
#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <Adafruit_NeoPixel.h>

#define PIXPIN 2 //production
//#define PIXPIN 1 //tester
#define NUMPIXELS 14
Adafruit_NeoPixel strip(NUMPIXELS, PIXPIN, NEO_GRB + NEO_KHZ800);
uint32_t red = strip.Color(255, 0, 0);
uint32_t green = strip.Color(0, 128, 0);
uint32_t blue = strip.Color(0, 0, 255);

#define SCREEN_WIDTH 128
#define SCREEN_HEIGHT 64
#define OLED_RESET     -1 
#define SCREEN_ADDRESS 0x3C

Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

#define YOUR_WIFI_SSID "********"
#define YOUR_WIFI_PASSWD "*******"

AlarmId id;

#define IR 13 //production
//#define IR 2 //tester
#define EXT 12

#define KEY "*********"
#define EVENT "dog_forgot"

Webhook webhook(KEY, EVENT);

int ampm = 0;
int skip = 0;
int remind = 0;
int detect;
int alreadyFed = 0;
unsigned long last_fed_time;
unsigned long last_fed_duration;
int fedAM = 0;
int fedPM = 0;
unsigned long open_duration;
const long blink_interval = 500;
unsigned long previousMillis = 0;

int hold_duration = 3; //seconds to count lid as opened


void setup() {
  Serial.begin(115200);
  Serial.println();

  pinMode(EXT, INPUT_PULLUP);
  pinMode(IR, INPUT);

  strip.begin();
  strip.clear();
  strip.fill(blue);
  strip.show();

  if(!display.begin(SSD1306_SWITCHCAPVCC, SCREEN_ADDRESS)) {
  Serial.println(F("SSD1306 allocation failed"));
  for(;;); // Don't proceed, loop forever
  }

  display.display();
  display.clearDisplay();
  display.setTextSize(2);             // Normal 1:1 pixel scale
  display.setTextColor(SSD1306_WHITE);        // Draw white text
  display.setCursor(0,0);             // Start at top-left corner
  display.println(F("CONNECTING"));
  display.display();


  WiFi.mode(WIFI_STA);
  WiFi.begin(YOUR_WIFI_SSID, YOUR_WIFI_PASSWD);

  configTime(0, 0, "0.us.pool.ntp.org");
  // America/New_York": "EST5EDT,M3.2.0,M11.1.0"
  setenv("TZ", "EST5EDT,M3.2.0,M11.1.0", 1);
  tzset();
  Serial.print("Clock before sync: ");
  digitalClockDisplay();
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.print("Clock after Wifi: ");
  
  display.clearDisplay();
  display.setTextSize(2);             // Normal 1:1 pixel scale
  display.setTextColor(SSD1306_WHITE);        // Draw white text
  display.setCursor(0,0);             // Start at top-left corner
  display.println(F("CONNECTED!"));
  display.display();

  delay(500);
  // create the alarms, to trigger at specific times
  Alarm.alarmRepeat(5,0,0, FoodAMset);  
  Alarm.alarmRepeat(11,0,0, FoodAMremind);  
  Alarm.alarmRepeat(17,0,0, FoodPMset);  
  Alarm.alarmRepeat(21,0,0, FoodPMremind);  


  // create timers, to trigger relative to when they're created
  // Alarm.timerRepeat(15, Repeats);           // timer for every 15 seconds
  // id = Alarm.timerRepeat(2, Repeats2);      // timer for every 2 seconds
  // Alarm.timerOnce(10, OnceOnly);            // called once after 10 seconds
  strip.clear();
  strip.show();
  display.clearDisplay();
  display.display();
}

void loop() {
  digitalClockDisplay();
  Alarm.delay(50);
  delay(50);
  scan();
  if (detect == 1) {
    unsigned long start_time = millis();
    while (detect == 1) {
      scan();
      delay(50);
      if (millis() - start_time > hold_duration * 1000) {
        hold();
        break;
      }
    }
    if (millis() - start_time <= hold_duration * 1000) {
      swipe();
    }
  }
  if (remind == 1) {
    forgot();
  }
}

// functions to be called when an alarm triggers:
void FoodAMset() {
  Serial.println("AM Food Armed");
  if (skip == 0) {
    ampm = 0;
    fedAM = 0;
    alreadyFed = 0;
    hungry();
  } else {
    fedAM = 1;
    alreadyFed = 1;
    skip = 0;
  }
}

void FoodPMset() {
  if (skip == 0) {
    ampm = 1;
    fedPM = 0;
    alreadyFed = 0;
    hungry();
  } else {
    fedPM = 1;
    alreadyFed = 1;
    skip = 0;
  }
}

void FoodAMremind() {
  if (fedAM == 0) {
    remind = 1;

    unsigned long elapsed_time = (millis() - last_fed_time) / 1000;
    unsigned long hours = elapsed_time / 3600;
    webhook.trigger(String(hours));
    forgot();
  }
}

void FoodPMremind() {
  if (fedPM == 0) {
    remind = 1;
    forgot();
  }
}

void scan() {
  detect = !digitalRead(IR); //inverted for normal use
}

void swipe() {
  // Show elapsed time on the screen since last fed time for a few seconds
  display.clearDisplay();
  display.setTextSize(2);
  display.setTextColor(SSD1306_WHITE);
  display.setCursor(0, 0);
  display.println("Last Fed");
  display.setTextSize(3);
  display.setCursor(0, 20);
  unsigned long elapsed_time = (millis() - last_fed_time) / 1000;
  unsigned long hours = elapsed_time / 3600;
  display.print(hours);
  display.println(" hours");
  display.display();
  delay(3000); // Show for 3 seconds
  display.clearDisplay();
  display.display();
}

void hold() {
  if (alreadyFed == 0) {
  while (detect == 1) {
      scan();
      delay(50);
    }
      fed();
  } else {
    while (detect == 1) {
      scan();
      delay(50);
    }
      fed2nd();
  }
}

void hungry() {
  Serial.println("Turning on LEDs");
  strip.clear();
  strip.fill(red);
  strip.show();
}

void forgot() {
  unsigned long currentMillis = millis();
  if (currentMillis - previousMillis >= blink_interval) {
    previousMillis = currentMillis;
    if (strip.getPixelColor(0) == red) {
      strip.clear();
    } else {
      strip.fill(red);
    }
    strip.show();
  }
}

void fed() {
  switch (ampm) {
    case 0:
    fedAM = 1;
    break;
    case 1:
    fedPM = 1;
    break;
  }
  strip.clear();
  strip.fill(green);
  strip.show();
  last_fed_time = millis();
  remind = 0;
  alreadyFed = 1;
}

void fed2nd() {
  // Display message asking if you want to skip next feeding
  
  // Start 30 second countdown
  unsigned long start_time = millis();
  unsigned long end_time = start_time + 30000; // 30 seconds
  skip = 0;
  
  while (millis() < end_time) {
    // Display countdown
    display.clearDisplay();
    display.setTextSize(2);
    display.setTextColor(SSD1306_WHITE);
    display.setCursor(0, 0);
    display.println("SKIP NEXT?");
    display.setCursor(0, 20);
    display.setTextSize(4);
    display.print((end_time - millis()) / 1000);
    display.display();
    
    // Cycle LEDs in a chase pattern
    for (int i = 0; i < NUMPIXELS; i++) {
      strip.clear();
      strip.setPixelColor(i, blue);
      strip.show();
      delay(50);
    }
    
    // Check if IR sensor is detected
    scan();
    if (detect == 1) {
      skip = 1;
      break;
    }
  }
  
  // Clear display and LEDs
  display.clearDisplay();
  display.display();
  strip.clear();
  strip.fill(green);
  strip.show();
}

void digitalClockDisplay() {
  time_t tnow = time(nullptr);
  Serial.println(ctime(&tnow));
  // display.clearDisplay();
  // display.setTextSize(1);
  // display.setTextColor(SSD1306_WHITE);
  // display.setCursor(0, 0);
  // display.println("Current Time:");

  // display.setTextColor(SSD1306_WHITE);        // Draw white text
  // display.setCursor(0,20);             // Start at top-left corner
  // display.println(ctime(&tnow));
  // display.display();

}
